#ifndef TARTANALOOP_HH
#define TARTANALOOP_HH

class TArtEventStore;
class TThread;
class TObject;
class TDirectory;
class TString;
class TCutG;
class TH1;

class TArtAnaLoop;
class TArtAnaFile;

//alias

TThread* book(TArtAnaLoop* analoop,
	      const char* anafilename = 0, const char* filename = 0,
	      int eventnumber = -1, TDirectory* directory = 0);
void start();
void stop();
void end();
void status();
TCutG* hcut(int id = 0, const char* filename = 0, const char* cutname = 0);

class TArtAnaLoopManager
{
  friend class TArtAnaLoop;

public:
  static TThread* Book(TArtAnaLoop* analoop,
		       const char* anafilename = 0, const char* filename = 0,
		       int eventnumber = -1, TDirectory* directory = 0);
  static void Start();
  static void Stop();
  static void End();
  static void Status();

  static TCutG* HCut(int id = 0, const char* filename = 0, const char* cutname = 0);

private:
  static TThread* fThread;

  enum EAnalysisState{INIT, RUNNING, SUSPENDED, END, ERROR};
  static EAnalysisState kAnalysisState;

private:
  TArtAnaLoopManager(){;}
  virtual ~TArtAnaLoopManager();

  ClassDef(TArtAnaLoopManager,1);
};

class TArtAnaLoopStatic
{
public:
  static void* AnaLoopStatic(void *);

  static void SetAnaLoop(TArtAnaLoop* analoop);
  static void SetFileName(const char* filename);
  static void SetEventNumber(int eventnumber);
  static void SetDirectory(TDirectory* directory);

  static TArtAnaLoop* GetAnaLoop();
  static const char* GetFileName();
  static int GetEventNumber();
  static TDirectory* GetDirectory();

private:
  static TArtAnaLoop* fAnaLoop;
  static TString fFileName;
  static int fEventNumber;

  static TDirectory* fDirectory;

private:
  TArtAnaLoopStatic(){;}
  virtual ~TArtAnaLoopStatic();

  ClassDef(TArtAnaLoopStatic,1);
};

class TArtAnaLoop
{
  friend class TArtAnaLoopStatic;
  friend class TArtAnaLoopManager;

public:
  TArtAnaLoop();
  virtual ~TArtAnaLoop();

  virtual bool OpenAnaFile(const char* anafilename);
  virtual void AnaLoop(const char* filename = 0, int eventnumber = -1);
  virtual const char* ClassName() const = 0;
  virtual int GetEventNumber() const {return fNumEvents;}

protected:
  virtual void Construct() = 0;
  virtual void Calculate() = 0;
  virtual void Destruct() = 0;

  TArtAnaFile* fAnaFile;

private:
  virtual void Wait();

  TArtEventStore *fEventStore;
  int fNumEvents;
};

#endif
