// handler of BigRIPS parameters

#ifndef _TARTBIGRIPSPARAMETERS_H_
#define _TARTBIGRIPSPARAMETERS_H_

#include <TROOT.h>
#include <TList.h>
#include <map>

class TXMLNode;
class TArtPPACPara;
class TArtPlasticPara;
class TArtICPara; 
class TArtFocalPlanePara; 
class TArtRIDFMap;

class TArtBigRIPSParameters : public TNamed
{
 public: 

  TArtBigRIPSParameters();
  ~TArtBigRIPSParameters();
  Bool_t LoadParameter(char *xmlfile);

  TList* GetListOfPPACPara()      {return listOfPPACPara; }
  TList* GetListOfPlasticPara()   {return listOfPlasticPara; }
  TList* GetListOfICPara()        {return listOfICPara; }
  TList* GetListOfFocalPlanePara(){return listOfFocalPlanePara;}

  Int_t GetNumPPACPara()      {return listOfPPACPara->GetEntries(); }
  Int_t GetNumPlasticPara()   {return listOfPlasticPara->GetEntries(); }
  Int_t GetNumICPara()        {return listOfICPara->GetEntries(); }
  Int_t GetNumFocalPlanePara(){return listOfFocalPlanePara->GetEntries();}

  TArtPPACPara * GetPPACPara(TArtRIDFMap *rmap);
  TArtPlasticPara * GetPlasticPara(TArtRIDFMap *rmap);
  TArtPlasticPara * GetPlasticPara(TString *name);
  TArtICPara * GetICPara(TArtRIDFMap *rmap);
  TArtFocalPlanePara * GetFocalPlanePara(int fpl);

  void   SetFocusPosOffset(Int_t fpl, Double_t offset);

 protected:

  std::map<TArtRIDFMap, TArtPPACPara *> ppac_pmap;
  std::map<TArtRIDFMap, TArtPlasticPara *> pla_pmap;
  std::map<TString, TArtPlasticPara *> pla_name_pmap;
  std::map<TArtRIDFMap, TArtICPara *> ic_pmap;
  std::map<int, TArtFocalPlanePara *> fpl_pmap;

  TList *listOfPPACPara;
  TList *listOfPlasticPara;
  TList *listOfICPara;
  TList *listOfFocalPlanePara;

  void ParseParaList(TXMLNode *node);
  TArtPPACPara * ParsePPACPara(TXMLNode *node);
  TArtPlasticPara * ParsePlasticPara(TXMLNode *node);
  TArtICPara * ParseICPara(TXMLNode *node);
  TArtFocalPlanePara * ParseFocalPlanePara(TXMLNode *node);

  ClassDef(TArtBigRIPSParameters,1);

};

#endif
